/*
 * SocksCleaner TS Services, Copyright (C) 2000 Nedelcho Stanev
 * This program comes with absolutely NO WARRANTY
 *
 * Should you choose to use and/or modify this source code, please
 * do so under the terms of the GNU General Public License under which
 * this program is distributed.
 *
 * $Id: conf.c,v 1.0.0 2000/03/03 22:17:23 decho Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <assert.h>
#include <netdb.h>

#include "alloc.h"
#include "conf.h"
#include "config.h"
#include "scdefs.h"
#include "log.h"
#include "match.h"
#include "misc.h"
#include "settings.h"
#include "sock.h"
#include "Strn.h"

static char *getfield (char *newline);

static void AddServ(char *hostname, char *password, int port);
static void ClearConfLines();

struct PortInfo *PortList = NULL;

static struct Servlist *CreateServer();
static void AddException(char *host);
#if 0
static void AddAddressToServer(struct Servlist **ptr,
                               char *ip, int length);
#endif /* 0 */

struct Servlist *ServList = NULL;           /* list of hub servers */
struct Exceptlist *ExceptionList = NULL;    /* list of exceptions */

int HubCount = 0;                    /* number of S: lines in sockscleaner.conf */

/* 
 * Global - pointer to structure containing the info for the hub
 *          services is currently connected to
 */
struct Servlist *currenthub = NULL;

/*
Rehash()
  args: none
  purpose: clear all old jupe, gline, and user lists and parse
           ConfigFile for new ones.
  return: 0 if successful rehash, 1 if not
*/

int
Rehash()

{
  int RehashError;
  FILE *fp;
  struct Servlist *tempserv;

  /*
   * We have to try to open the configuration file before
   * calling ParseConf() in case we no longer have access to
   * it, or it is no longer there, in which case do not
   * kill the old conf lists. Unfortunately this means we'll
   * have to call fopen() on it twice (once again in ParseConf()).
   */
  if (!(fp = fopen(ConfigFile, "r")))
    return (1);

  fclose(fp);

  /* Kill old  conf lists */
  ClearConfLines();

  /*
   * mark servers with delete flag and undelete them as we read
   * in S: lines from conf - if we clear the ServList, currenthub
   * will point to garbage, so this makes sure that doesn't happen
   */
  for (tempserv = ServList; tempserv; tempserv = tempserv->next)
    tempserv->flags |= SL_DELETE;

  HubCount = 0;

  /*
   * Reload configuration files
   */
  ParseConf(ConfigFile, 1);

  RehashError = 0;


  if (!ServList)
  {
    RehashError = 1;
    putlog(LOG1, "Rehash: No S: lines found in %s",
      ConfigFile);
  }


  /*
   * Rehash settings.conf
   */
  if (LoadSettings(1) == 0)
    RehashError = 1;

  return (RehashError);
} /* Rehash() */

/*
ParseConf()
 Backend for LoadConfig().
 Args: filename - file name to parse
       rehash - set to 1 if this is a rehash
*/

void
ParseConf(char *filename, int rehash)

{
  FILE *fileptr;
  char line[MAXLINE];
  char *key;
  int scnt; /* S: line count */

  if (!filename)
    return;

  if ((fileptr = fopen(filename, "r")) == NULL)
  {
    fprintf(stderr, "Unable to open %s\n", filename);
    putlog(LOG1,
      "Unable to open configuration file: %s %s",
      filename,
      (filename == ConfigFile) ? "(FATAL)" : "");

    /*
     * If the file is the top-level config file, exit.
     * If this is a rehash, do not exit, because Rehash()
     * would NOT have erased the old conf lists, so we
     * can continue normally.
     */
    if (!rehash && (filename == ConfigFile))
      exit(0);

    return;
  }

  scnt = 0;

  while (fgets(line, MAXLINE - 1, fileptr))
  {
    /*
     * you may add comments to config file using a '#'
     */
    if (line[0] == '#')
      continue;         

    key = getfield(line);
    if (!key)
      continue;

    switch (*key)
    {
      case 'a':
      case 'A':
      {
        char *temp;

        /*
         * If this is a rehash, Me.admin should already be
         * set, so make sure it gets freed.
         */
        if (Me.admin)
          MyFree(Me.admin);

        if (!(temp = getfield(NULL)))
          break;

        if (strlen(temp) > REALLEN)
        {
          Me.admin = (char *) MyMalloc(REALLEN);
          memset(Me.admin, 0, REALLEN);
          Strncpy(Me.admin, temp, REALLEN - 1);
        }
        else
          Me.admin = MyStrdup(temp);

        break;
      } /* case 'A' */

      case 's':
      case 'S':
      {
        char *pass;
        char *host;
        char *port;
        struct Servlist *servptr;
        
        pass = getfield(NULL);
        host = getfield(NULL);
        port = getfield(NULL);
        if (!host || !pass)
          continue;

        if ((servptr = IsServLine(host, port)))
        {
          servptr->flags &= ~SL_DELETE;
          if (servptr->password)
          {
            /*
             * This must be a rehash, so in case the password
             * was changed, set servptr->password to the new one.
             */
            MyFree(servptr->password);
            servptr->password = MyStrdup(pass);
          }
        }
        else
        {
          AddServ(host, pass, port ? atoi(port) : 6667);
        }

        scnt++;
        break;
      } /* case 'S' */
      case 'e':
      case 'E':
      {
        char *host;

        host = getfield(NULL);
        if (!host)
          continue;

        AddException(host);

        break;
      } /* case 'O' */

      case 'n':
      case 'N':
      {
        char *name;
        char *info;

        name = getfield(NULL);
        info = getfield(NULL);
        if (!name || !info)
          continue;

        if (strlen(name) > REALLEN)
        {
          Me.name = (char *) MyMalloc(REALLEN);
          memset(Me.name, 0, REALLEN);
          Strncpy(Me.name, name, REALLEN - 1);
        }
        else
          Me.name = MyStrdup(name);

        if (strlen(info) > REALLEN)
        {
          Me.info = (char *) MyMalloc(REALLEN);
          memset(Me.info, 0, REALLEN);
          Strncpy(Me.info, info, REALLEN - 1);
        }
        else
          Me.info = MyStrdup(info);

        break;
      } /* case 'N' */

      case 'v':
      case 'V':
      {
        char *vhost;

        vhost = getfield(NULL);
        if (!vhost)
          continue;

        if (!LocalHostName)
          LocalHostName = MyStrdup(vhost);

        break;
      } /* case 'V' */

      case '.':
      {
        char *start, *end;
        char *filename;

        if (!strncasecmp(line, ".include", 8))
        {
          /*
           * It is a .include statement - meaning they
           * want to include another file to be parsed
           * as a regular configuration file. Normal
           * syntax is: .include "filename"
           */

          if ((start = strchr(line, '"')))
            filename = start + 1;
          else
          {
            /*
             * There are no quotes in the line - bad syntax
             */
            putlog(LOG1,
              "Incorrect config file syntax: %s",
              line);
            continue;
          }

          /*
           * "filename" should now point past the first quote
           * (ie: file") so now set the end quote to a \0
           */
          if ((end = strchr(filename, '"')))
            *end = '\0';
          else
          {
            putlog(LOG1,
              "Incorrect config file syntax: %s",
              line);
            continue;
          }

          /*
           * Now recursively call ParseConf() to parse the new
           * configuration file
           */
          ParseConf(filename, 0);
        } /* if (!strncasecmp(line, ".include", 8)) */

        break;
      } /* case '.' */

      default:
        break;
    }
  }

  HubCount += scnt;

  fclose(fileptr);
} /* ParseConf() */

/*
LoadConfig()
  args: none
  purpose: parse config file and store server, admin, user, and 
           jupe/gline information
  return: none
*/

void
LoadConfig()

{
  int ConfError;

  ParseConf(ConfigFile, 0);

  ConfError = 0;

  if (!Me.admin)
  {
    fprintf (stderr,
      "No administrative contact information (A:) found in %s\n",
      ConfigFile);
    ConfError = 1;
  }

  if (!Me.name)
  {
    fprintf(stderr,
      "No server name (N:) specified in %s\n",
      ConfigFile);
    ConfError = 1;
  }

  if (!Me.info)
  {
    fprintf (stderr,
      "No server info (N:) specified in %s\n",
      ConfigFile);
    ConfError = 1;
  }

  if (!ServList)
  {
    fprintf (stderr,
      "No hub servers (S:) specified in %s\n",
      ConfigFile);
    ConfError = 1;
  }
  
  if (ConfError)
    exit(1);
} /* LoadConfig() */

/*
getfield()
  Get next field separated by a : in the conf file
  ** This function is from standard ircd source
*/

static char *
getfield (char *newline)

{
  static char *line = NULL;
  char *end, *field;
  
  if (newline)
    line = newline;

  if (!line)
    return (NULL);

  field = line;
  if ((end = strchr(line, ':')) == NULL)
  {
    line = (char *) NULL;
    if ((end = strchr(field, '\n')) == NULL)
      end = field + strlen(field);
  }
  else
    line = end + 1;

  *end = '\0';
  if (strlen(field))
    return (field);
  else
    return (NULL);
} /* getfield() */

/*
IsServLine()
  args: char *name, int port
  purpose: determine if there is an S: line for 'name' and 'port'
  return: pointer to server struct in server list
*/

struct Servlist *
IsServLine(char *name, char *port)

{
  struct Servlist *tempserv;
  int portnum;

  if (!name)
    return (NULL);

  if (!port)
    portnum = 6667;
  else
    portnum = atoi(port);

  for (tempserv = ServList; tempserv; tempserv = tempserv->next)
    if ((tempserv->port == portnum) &&
        (!strcasecmp(tempserv->hostname, name)))
      return (tempserv);

  return (NULL);
} /* IsServLine() */

/*
AddServ()
  Add 'hostname', 'password', and 'port' to list containing S: line
info
*/

static void
AddServ(char *hostname, char *password, int port)

{
  struct Servlist *ptr;

#if 0
  /*
   * maybe someday when I'm really bored I'll fix this
   */
  struct hostent *hp;
  struct in_addr ip;
  char **curraddr;

  /*
   * All S: line hostnames are pre-resolved so when we
   * want to connect to a server, we don't waste time resolving
   * the hostname again and again.
   */
  hp = LookupHostname(hostname, &ip);

  if (hp)
  {
    /*
     * We have a resolving hostname - it is possible it points
     * to more than one ip - add all of them
     */
    ptr = CreateServer();

    for (curraddr = hp->h_addr_list; *curraddr; ++curraddr)
      AddAddressToServer(&ptr, *curraddr, hp->h_length);
  }
  else
  {
    if (ip.s_addr == INADDR_NONE)
    {
      putlog(LOG1,
        "Unresolvable server name: %s",
        hostname);
      return;
    }

    ptr = CreateServer();
    AddAddressToServer(&ptr, (char *) &ip.s_addr, sizeof(ip));
  }
#endif /* 0 */

  ptr = CreateServer();
  ptr->hostname = MyStrdup(hostname);
  ptr->password = MyStrdup(password);
  ptr->port = port;
} /* AddServ() */

/*
CreateServer()
 Allocate memory for a new Servlist structure and return pointer
*/

static struct Servlist *
CreateServer()

{
  struct Servlist *ptr;

  ptr = (struct Servlist *) MyMalloc(sizeof(struct Servlist));
  memset(ptr, 0, sizeof(struct Servlist));

  ptr->sockfd = NOSOCKET;

  ptr->next = ServList;
  ServList = ptr;

  return (ptr);
} /* CreateServer() */

#if 0
/*
AddAddressToServer()
 Add the given ip address to the given server structure
*/

static void
AddAddressToServer(struct Servlist **ptr, char *ip, int length)

{
  int offset;

  ++(*ptr)->numips;
  (*ptr)->ips = (struct in_addr **)
    MyRealloc((*ptr)->ips, sizeof(struct in_addr) * (*ptr)->numips);

  offset = sizeof(struct in_addr) * ((*ptr)->numips - 1);
  memcpy((*ptr)->ips[0] + offset, ip, length);
} /* AddAddressToServer() */
#endif /* 0 */

/*
ClearConfLines()
 Called from Rehash() to clear old conf list
*/

static void
ClearConfLines()

{
/*  struct Servlist *tempserv; */
    struct Exceptlist *templist;

/*
 * Don't clear the ServList during a rehash because currenthub
 * will point to garbage
  while (ServList)
  {
    MyFree(ServList->hostname);
    MyFree(ServList->password);
    if (ServList->realname)
      MyFree(ServList->realname);
    tempserv = ServList->next;
    MyFree(ServList);
    ServList = tempserv;
  }
*/
  while (ExceptionList)
  {
    templist = ExceptionList->next;
    MyFree(ExceptionList->hostname);
    MyFree(ExceptionList);
    ExceptionList = templist;
  }

} /* ClearConfLines() */

static void
AddException(char *host)

{
  struct Exceptlist *ptr;

  ptr = (struct Exceptlist *) MyMalloc(sizeof(struct Exceptlist));
  memset(ptr, 0, sizeof(struct Exceptlist));
  ptr->hostname = MyStrdup(host);
  ptr->next = ExceptionList;
  ExceptionList = ptr;
  putlog( LOG1, "Added exception:%s", host );
} /* AddException() */


/*
IsProtectedHost()
  Determines if userhost matches an exception E: line -
returns 1 if so, 0 if not
*/

int
IsProtectedHost(char *hostname)

{
  struct Exceptlist *templist;

  if (!hostname)
    return 0;

  for (templist = ExceptionList; templist; templist = templist->next)
  {
    if (match(templist->hostname,hostname) ) 
      return 1;
  }

  return 0;
} /* IsProtectedHost() */
