/*
 * SocksCleaner TS Services, Copyright (C) 2000 Nedelcho Stanev
 * This program comes with absolutely NO WARRANTY
 *
 * Should you choose to use and/or modify this source code, please
 * do so under the terms of the GNU General Public License under which
 * this program is distributed.
 *
 * $Id: init.c,v 1.0.0 2000/03/03 22:17:23 decho Exp $
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <string.h>

#include "alloc.h"
#include "client.h"
#include "conf.h"
#include "config.h"
#include "defs.h"
#include "scdefs.h"
#include "log.h"
#include "misc.h"
#include "mystring.h"
#include "settings.h"
#include "sock.h"
#include "timer.h"

#ifdef HAVE_PTHREADS
#include <pthread.h>
#endif

/*
ProcessSignal()
  args: int sig
  purpose: process system signal specified by 'sig'
  return: none
*/

void
ProcessSignal(int sig)

{
  switch (sig)
  {
    /* rehash configuration file */
    case SIGHUP:
    {
      putlog(LOG1,
        "Received signal SIGHUP, rehashing configuration file and databases");

      Rehash();

      signal(SIGHUP, ProcessSignal); /* reset the signal */
      break;
    }

    /* restart services */
    case SIGINT:
    {
    #ifdef DEBUGMODE
      exit(1);
    #else
      putlog(LOG1, "Received signal SIGINT, restarting");
      signal(SIGINT, ProcessSignal);
    #endif
      break;
    }

    case SIGPIPE:
    {
      putlog(LOG1, "Received signal SIGPIPE, ignoring");
      /*abort();*/
      break;
    }

    /* 
     * this is required to prevent a child process from becoming a
     * zombie which just sits out there taking up fds
     */
    case SIGCHLD:
    {
      wait(NULL);
      signal(SIGCHLD, ProcessSignal);
      break;
    }

    /* something really died */
    case SIGSEGV:
    case SIGBUS:
    case SIGTERM:
    {
      if (sig == SIGSEGV)
      {
        toserv(":%s QUIT :%s\n:%s",
          "SocksCleaner",
          "ACK! SIGSEGV!");
        putlog(LOG1,
          "Received signal SIGSEGV, shutting down (databases not saved)");
      }
      else
        putlog(LOG1, "Received signal %s, shutting down",
          (sig == SIGBUS) ? "SIGBUS" : "SIGTERM");

      if (sig != SIGTERM)
        abort(); /* make a core file to work with */

      if (sig == SIGSEGV)
      {
        /*
         * Don't write databases on SIGSEGV, because there's
         * a good chance they could get corrupted
         */
        /* DoShutdown((char *) NULL, "SIGSEGV Received"); */
        exit(1);
      }
      else
      {
        DoShutdown((char *) NULL, "SIGTERM Received");
      }
    }
  }
} /* ProcessSignal() */


/*
InitSignals()
 Initialize signal hooks
*/

void
InitSignals()

{
#ifdef HAVE_PTHREADS
  pthread_attr_t attr;
  pthread_t signalid;
#endif

#ifdef HAVE_PTHREADS

  /* pthreads and signal() do not go well together */
  pthread_attr_init(&attr);
  pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
  pthread_create(&signalid, &attr, p_CheckSignals, NULL);

#else

  /* setup signal hooks */
  signal(SIGHUP, ProcessSignal);
  signal(SIGSEGV, ProcessSignal);
  signal(SIGBUS, ProcessSignal);
  signal(SIGTERM, ProcessSignal);
  signal(SIGINT, ProcessSignal);
  signal(SIGCHLD, ProcessSignal);
  signal(SIGPIPE, SIG_IGN);

#endif /* HAVE_PTHREADS */

} /* InitSignals() */

/*
PostCleanup()
  Called after we are disconnected from a hub server, to
reset some linked lists
*/

void
PostCleanup()

{
  Me.sptr = NULL;
  Me.hub = NULL;

} /* PostCleanup() */

/*
InitServs()
  purpose: introduce *SERV_NICKs to the network after a successful
           connection - if servptr specified, introduce only
           that service
  return: none
*/

void
InitServs()

{
   char nick[NICKLEN+1];
   char ident[NICKLEN+1];
   char desc[USERLEN];
   char sendstr[MAXLINE];
   time_t CurrTime = time(NULL);
 
   memset( &nick, 0x00, NICKLEN+1 );
   memset( &ident, 0x00, NICKLEN+1 );
   memset( &desc, 0x00, USERLEN );

   sprintf( nick, CleanerName );
   sprintf( ident,CleanerName );
   sprintf( desc, "Socks monitor ");

   putlog( LOG1, "Let's introduce nicks to server");
   putlog( LOG1, "Notify msg:%s", CheckNotifyMsg );
   putlog( LOG1, "KillSocks:%d GlineSocks:%d SafeConnect:%d", KillSocks, GlineSocks, SafeConnect );
#ifdef USE_SERVICES
   putlog( LOG1, "Services hostname:%s", ServicesHost );
#endif 
   putlog( LOG1, "KillReason:%s", KillReason ); 
   putlog( LOG1, "GlineReason:%s", GlineReason );
   putlog( LOG1, "Enable monitoring:%d" , EnableMonitoring );
   putlog( LOG1, "Monitoring channel:%s", MonitoringChannel );
   putlog( LOG1, "Execption list:" );

#ifdef BAHAMUT_DAEMON
    sprintf(sendstr, "NICK %s 1 %ld +iw %s %s %s 1 :%s\n",
       nick,
       (long) CurrTime,
       ident,
       Me.name,
       Me.name,
       desc);
#else
    sprintf(sendstr, "NICK %s 1 %ld +iw %s %s %s :%s\n",
       nick,
       (long) CurrTime,
       ident,
       Me.name,
       Me.name,
       desc);
#endif
    toserv(sendstr);
   if( EnableMonitoring ) {
     sprintf(sendstr,
        ":%s SJOIN %ld %s + :@%s\n",
        Me.name,
        (long) CurrTime,
        MonitoringChannel,
        CleanerName );
      toserv(sendstr);
   }
  
} /* InitServs() */
