/*
 * SocksCleaner TS Services, Copyright (C) 2000 Nedelcho Stanev
 * This program comes with absolutely NO WARRANTY
 *
 * Should you choose to use and/or modify this source code, please
 * do so under the terms of the GNU General Public License under which
 * this program is distributed.
 *
 * $Id: misc.c,v 1.0.0 2000/03/03 22:17:23 decho Exp $
 */

#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <assert.h>

#include "alloc.h"
#include "conf.h"
#include "config.h"
#include "defs.h"
#include "scdefs.h"
#include "log.h"
#include "match.h"
#include "misc.h"
#include "settings.h"
#include "sock.h"
#include "Strn.h"

extern char *crypt();

/*
 * This is an array of all the service bots. It's useful
 * because it keeps code repitition down. For example, when
 * we want to introduce all the service nicks to the network
 * upon connection, we can just loop through this array, rather
 * than writing out all the introduce() calls. It's also useful
 * for the remote trace and whois queries.
 */


/*
debug()
  Output a debug message to stderr
*/

void
debug(char *format, ...)
#ifdef DEBUGMODE
{
  va_list args;

  va_start(args, format);

  fprintf(stderr, "DEBUG> ");
  vfprintf(stderr, format, args);

  va_end(args);
} /* debug() */
#else
{
 ;
}
#endif

/*
fatal()
  Fatal error occured - log it and exit
*/

void
fatal(int keepgoing, char *format, ...)

{
  char buf[MAXLINE];
  va_list args;
  int oldlev;

  va_start(args, format);

  vSnprintf(buf, sizeof(buf), format, args);

  va_end(args);

  /*
   * If this is being called from dparse(), it's possible
   * LogLevel hasn't been set yet. So, make sure this
   * error gets logged, whether LogLevel is set or not
   */
  oldlev = LogLevel;
  LogLevel = 1;

  /* log the error */
  if (!keepgoing)
  {
    putlog(LOG1, "FATAL: %s", buf);
    exit(1);
  }
  else
    putlog(LOG1, "Warning: %s", buf);

  LogLevel = oldlev;
} /* fatal() */

/*
notice()
  purpose: send a NOTICE to 'nick' with 'msg'
  return: none
*/

void
notice(char *from, char *nick, char *format, ...)

{
  char finalstr[MAXLINE];
  char who[MAXLINE];
  va_list args;

  va_start(args, format);

  vSnprintf(finalstr, sizeof(finalstr), format, args);

  strcpy(who, from);

  toserv(":%s NOTICE %s :%s\n", 
    who,
    nick,
    finalstr);

  va_end(args);
} /* notice() */

/*
DoShutdown()
  Shut down services
*/

void
DoShutdown(char *who, char *reason)

{
  char sendstr[MAXLINE];

  putlog(LOG1, "Shutting down services");

  if (reason)
  {
    if (who)
      sprintf(sendstr, "%s (authorized by %s)", reason, who);
    else
      strcpy(sendstr, reason);
  }
  else
    if (who)
      sprintf(sendstr, "Authorized by %s", who);
    else
      sendstr[0] = '\0';

  toserv(":%s QUIT :%s\nSQUIT %s :%s\n",
    CleanerName,
    "Shutting Down",
    currenthub->realname ? currenthub->realname : currenthub->hostname,
    sendstr);

  exit (1);
} /* DoShutdown() */

/* 
HostToMask()
  Convert a userhost like "user@host.name.org" into "*user@*.name.org"
or "user@1.2.3.4" into "*user@1.2.3.*"
*/

char *
HostToMask (char *username, char *hostname)

{
  char *realhost, /* user@host (without nick!) */
       *temp,
       *final, /* final product */
       *user, /* stores the username */
       *host, /* stores the hostname */
       /* 
        * first segment of hostname (ie: if host == "a.b.c",
        * topsegment == "c" (top-level domain)
        */
       *topsegment = NULL;
  char userhost[UHOSTLEN + 2];
  int ii, /* looping */
      len;

  if (!username || !hostname)
    return ((char *) NULL);

  sprintf(userhost, "%s@%s",
    username,
    hostname);

  len = strlen(userhost) + 32;

  final = (char *) MyMalloc(len);
  memset(final, 0, len);

  /* strip off a nick nick!user@host (if there is one) */
  realhost = (host = strchr(userhost, '!')) ? host + 1 : userhost;
  user = realhost;
  if ((host = strchr(realhost, '@')))
  {
    final[0] = '*';
    ii = 1;
    /* 
     * only use the last 8 characters of the username if there 
     * are more
     */
    if ((host - realhost) > 10)
      user = host - 8;

    /* now store the username into 'final' */
    while (*user != '@')
      final[ii++] = *user++;

    final[ii++] = '@';

    /* host == "@host.name", so point past the @ */
    host++;
  }
  else
  {
    /* there's no @ in the hostname, just make it *@*.host.com */
    strcpy(final, "*@");
    ii = 2;
    host = userhost;
  }

  /* 
   * ii now points to the offset in 'final' of where the 
   * converted hostname should go
   */

  realhost = strchr(host, '.');
  if (realhost)
    topsegment = strchr(realhost + 1, '.');
  if (!realhost || !topsegment)
  {
    /* 
     * if realhost is NULL, then the hostname must be a top-level
     * domain; and if topsegment is NULL, the hostname must be
     * 2 parts (ie: blah.org), so don't strip off "blah"
     */
    strcpy(final + ii, host);
  }
  else
  {
    /*
     * topsegment now contains the top-level domain - if it's
     * numerical, it MUST be an ip address, since there are
     * no numerical TLD's =P
     */

    /* advance to the end of topsegment */
    for (temp = topsegment; *temp; temp++);

    --temp; /* point to the last letter (number) of the TLD */
    if ((*temp >= '0') && (*temp <= '9'))
    {
      /* Numerical IP Address */
      while (*temp != '.')
        --temp;

      /* 
       * copy the ip address (except the last .XXX) into the 
       * right spot in 'final'
       */
      Strncpy(final + ii, host, temp - host);

      /* stick a .* on the end :-) */
      ii += (temp - host);
      strcpy(final + ii, ".*");
    }
    else
    {
      /* its a regular hostname with >= 3 segments */

      /*
       * topsegment doesn't necessarily point to the TLD.
       * It simply points one segment further than realhost.
       * Check if there is another period in topsegment,
       * and if so use it. Otherwise use realhost
       */
      sprintf(final + ii, "*%s", 
        strchr(topsegment + 1, '.') ? topsegment : realhost);
    }
  }

  return (final);
} /* HostToMask() */


/*
pwmatch()
  Check if 'password' matches 'chkpass' - return 1 if so, 0 if not
Assumes you're trying to match passwords from *Serv databases
*/

int
pwmatch(char *password, char *chkpass)

{
#ifdef CRYPT_PASSWORDS
  char *encr;
#endif

  if (!password || !chkpass)
    return 0;

#ifdef CRYPT_PASSWORDS

  encr = (char *) crypt(chkpass, password);
  assert(encr != 0);

  if (!strcmp(encr, password))
    return 1;
  else
    return 0;

#else

  if (!strcmp(password, chkpass))
    return 1;
  else
    return 0;

#endif
} /* pwmatch() */

/*
operpwmatch()
  Check if 'password' matches 'chkpass' - return 1 if so, 0 if not
Assumes you're trying to match passwords from O: lines in sockscleaner.conf
*/

int
operpwmatch(char *password, char *chkpass)

{
#ifdef CRYPT_OPER_PASSWORDS
  char *encr;
#endif

  if (!password || !chkpass)
    return 0;

#ifdef CRYPT_OPER_PASSWORDS

  encr = (char *) crypt(chkpass, password);
  assert(encr != 0);

  if (!strcmp(encr, password))
    return 1;
  else
    return 0;

#else

  if (!strcmp(password, chkpass))
    return 1;
  else
    return 0;

#endif
} /* operpwmatch() */


int
IsNum(char *str)

{
  int result = 0;
  char tmp[MAXLINE];
  char *tmp2;

  if (!str)
    return 0;

  strcpy(tmp, str);
  tmp2 = tmp;
  while (*tmp2)
  {
    if (IsDigit(*tmp2))
    {
      result *= 10;
      result += ((*tmp2) & 0xF);
      tmp2++;
    }
    else
      return 0;
  }

  /* if 'str' actually contains the number 0, return 1 */
  if (result == 0)
    result = 1;

  return (result);
} /* IsNum() */

	
