/*
 * SocksCleaner TS Services, Copyright (C) 2000 Nedelcho Stanev
 * This program comes with absolutely NO WARRANTY
 *
 * Should you choose to use and/or modify this source code, please
 * do so under the terms of the GNU General Public License under which
 * this program is distributed.
 *
 * $Id: server.c,v 1.0.0 2000/03/03 22:17:23 decho Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "alloc.h"
#include "conf.h"
#include "client.h"
#include "config.h"
#include "defs.h"
#include "scdefs.h"
#include "init.h"
#include "log.h"
#include "match.h"
#include "misc.h"
#include "mystring.h"
#include "proxy.h"
#include "server.h"
#include "settings.h"
#include "sock.h"
#include "Strn.h"
#include "timestr.h"


#ifdef HAVE_PTHREADS
#include <pthread.h>
#endif

/*
 * Global - list of network servers
 */
struct Server *ServerList = NULL;

static void s_ping(int ac, char **av);
static void s_kill(int sc, char **av);
static void s_nick(int ac, char **av);
static void s_error(int ac, char **av);
static void s_version(int ac, char **av);
static void s_server(int ac, char **av);
static void s_privmsg( int ac, char **av);

/* external */
int iGlined;
int iChecked;
int iProtected; 
int iThreads;

void onctcp( char* who, char* command);
void onstats( char* who);
void oncheck( char* who, char* command);
int userstrcnt = 0;
_UserStr *userstruct = NULL;

/*
 * Table of commands the hub will send us
 */
static struct ServCommand servtab[] = {
  { "PING", s_ping },
  { "NICK", s_nick },
  { "KILL", s_kill },
  { "privmsg", s_privmsg },
  { "ERROR", s_error },
  { "SERVER", s_server},
  { "VERSION", s_version },
  { 0, 0 }
};

/*
ProcessInfo()
  args: int arc, char **arv
  purpose: process string sent by hub server (stored in 'arv')
  return: none
*/

void
ProcessInfo(int arc, char **arv)

{
  struct ServCommand *stptr;
  int index;

  if ((arc >= 2) && (*arv[0] == ':'))
    index = 1;
  else
    index = 0;

  for (stptr = servtab; stptr->cmd; stptr++)
  {
    if (!strcasecmp(arv[index], stptr->cmd))
    {
      (*stptr->func)(arc, arv);
      break;
    }
  }
} /* ProcessInfo() */



/*
s_ping()
 Respond to a ping request

 If a server is requesting a PING
  av[0] = "PING"
  av[1] = :servername

 If a client is requesting a PING
  av[0] = :nickname
  av[1] = "PING"
  av[2] = nickname
  av[3] = Me.name
*/

static void
s_ping(int ac, char **av)

{
  char *who;

  if (ac < 2)
    return;

  if (!strcasecmp(av[0], "PING"))
    who = av[1];
  else
    who = av[0];

  if (*who == ':')
    who++;

  toserv(":%s PONG %s :%s\n", 
    Me.name, 
    Me.name, 
    who);

} /* s_ping() */


/*
s_nick()
  Process a NICK statement

  if new user:
  av[1] = nick name
  av[2] = hop count
  av[3] = TimeStamp
  av[4] = usermodes
  av[5] = username
  av[6] = hostname
  av[7] = server
  av[8] = real name

  if nick change:
  av[0] = who
  av[1] = "NICK"
  av[2] = new nick
  av[3] = TS
*/

static void
s_nick(int ac, char **av)

{
  struct Luser lptr;

#ifdef HAVE_PTHREADS
      pthread_t clientid;
//      pthread_attr_t attr;
#endif

  memset( &lptr   , 0x00, sizeof(lptr)  );

  if (ac == 4)
  {
    return;
  } /* if (ac == 4) */

  /* It is a new user */

  if (ac < 9)
    return;
  if( ProxyCheck && SafeConnect && SplitBurst ) {

#ifdef USE_SERVICES

/*
  Skip all nicks coming from UniBg.Services,
  this will skip connection to enforced nicks from services
*/
    if( strncasecmp( av[6], ServicesHost, strlen(ServicesHost)) == 0 ||
        strncasecmp( av[6], "juped.com", 9 ) == 0  ) {
#ifdef DEBUGMODE2
       putlog( LOG1, "Nick:%s with host:%s skipped!", av[1], av[6] );
#endif
       return;
    }
#endif

    strcpy(lptr.nick     , MyStrdup(av[1]));
    strcpy(lptr.username , MyStrdup(av[5]));
    strcpy(lptr.hostname , MyStrdup(av[6]));

#ifdef METHOD2
    check_add(av[1], av[5], av[6] );
#else
#ifdef HAVE_PTHREADS
      pthread_create(&clientid, NULL, (void *) &Proxy, (void *) &lptr);
      pthread_detach(clientid);
      pthread_join(clientid,NULL);
      usleep(10);
#endif /* HAVE_PTHREADS */
#endif /* METHOD2 */
  }
} /* s_nick() */



/*
s_error()
  Log the error that the hub sends
*/

static void
s_error(int ac, char **av)

{
  char *serror;

  if (ac >= 3)
    serror = av[2] + 1;
  else if (ac >= 2)
    serror = av[1] + 1;
  else
    serror = "";

  putlog(LOG1, "Server Error: %s", serror);
} /* s_error() */


/*
s_version()
  Reply to a remote version query

  av[0] = who
  av[1] = "VERSION"
  av[2] = server name
*/

static void
s_version(int ac, char **av)

{
  char *who;

  if (ac < 3)
    return;

  if (strcasecmp(Me.name, (av[2][0] == ':') ? av[2] + 1 : av[2]) != 0)
    return;

  if (av[0][0] == ':')
    who = av[0] + 1;
  else
    who = av[0];

  toserv(":%s 351 %s SocksCleaner-%s. %s :TS3 by Nedelcho Stanev <decho@iname.com>\n",
    Me.name,
    who,
    hVersion,
    Me.name);
} /* s_version() */

static void
s_server( int ac, char **av )
{

  char buf[256];
  char buf1[16];

  memset( &buf , 0x00, sizeof( buf ) );
  memset( &buf1, 0x00, sizeof( buf1) );
  if( ac < 4 )
    return;
  if( !strcasecmp( av[1], currenthub->hostname ))
    InitServs(NULL);

  if( SafeConnect ) {

    SplitBurst = 0;
    TimeLastConnected = time(NULL);

    if( EnableMonitoring ) {
           sprintf( buf, "New server : %s.Changing last connection time to : %ld " , av[2] ,TimeLastConnected );  
         toserv(":%s PRIVMSG %s :%s\n",
              CleanerName,
              MonitoringChannel,
              buf );
    } /* Enable monitoring */

  } /* Safe Connect */

}

static void
s_kill( int ac, char **av )
{
  if( !strcasecmp( av[2], CleanerName ))
    InitServs(NULL);
  if (ac < 4)
    return;

}

static void 
s_privmsg( int ac, char **av )
{
  char *who, /* nick who gave privmsg */
       *command, /* contains the /msg that 'who' gave */
       *tmp;  
  if (ac < 4)
    return;
  if (av[0][0] == ':')
    who = av[0] + 1;
  else
    who = av[0];
  if ((tmp = strchr(av[2], '@')))
  {
    if (!strcasecmp(tmp + 1, Me.name))
      *tmp = '\0';
  }
  
  command = av[3] + 1;
  putlog( LOG1, "target:%s command:%s from:%s", av[2], command, who );
  if( !strcasecmp( av[2], CleanerName ) && (command[0] == '\001')) {
    onctcp(who, command );
  }
#undef USER_COMMANDS
#ifdef USER_COMMANDS
  if( !strcasecmp( command, "STATS" )) {
    onstats(who);
  }

  if( !strncasecmp( command, "CHECK",5 )) {
       strtok( command, " " );	
       oncheck( who,strtok( NULL ," " ) );
  }
#endif

}

/*
onctcp()
  args: char *nick, char *msg
  purpose: handle ctcp message 'msg' sent by 'nick'
  return: none
*/

void
onctcp( char* who, char* command)
{

  if (!command)
    return;

  if (!*++command)
    return;

  if (!strncasecmp(command, "ACTION", 6))
    return; /* don't log ctcp actions */

  if (strncasecmp(command, "VERSION", 7) == 0)
  {
    notice(CleanerName, who,
      "\001VERSION SocksCleaner TS Services version %s\001",
      hVersion); 
    notice(CleanerName, who,
      "\001VERSION by Nedelcho Stanev <decho@iname.com>\001");
    return;
  }

  if (strncasecmp(command, "PING", 4) == 0)
  {
    notice(CleanerName, who, command - 1);
    return;
  }
}

void check_add(char *nick, char *user, char *host)
{

  if( if_exist_host( host )) {
    putlog( LOG1, "Host:%s exist, skipped new nick:%s", 
          host,
          nick );
  }
  else {
    _UserStr *strU = MyMalloc(sizeof(_UserStr));
    userstrcnt++;
    bzero(strU, sizeof(_UserStr));
    strncpy(strU->nick, nick, NICKLEN);
    strU->userhost = MyStrdup(host);
    strU->type = 1;
    strU->prev = userstruct; strU->next = NULL;
    if (userstruct) 
       userstruct->next = strU;
    userstruct = strU;
    putlog( LOG1, "CNT:%d Nick:%s Host:%s", 
       userstrcnt,
       userstruct->nick,
       userstruct->userhost );
  }
}

int if_exist_host( char* host )
{

  _UserStr* tmpstr;  
  
  for (tmpstr = userstruct; tmpstr; tmpstr = tmpstr->next) 
     if ( !strcasecmp( tmpstr->userhost, host ) )
        return 1;

  return 0;
}

void check_del( char* host )
{
   _UserStr* tmpstr;

   if( !host )
     return;

   putlog( LOG1, "Look for host:%s to delete it!", host );

   for (tmpstr = userstruct; tmpstr; tmpstr = tmpstr->next)
     if ( !strcasecmp( tmpstr->userhost, host ) ) 
     {
         if (tmpstr->prev)
           tmpstr->prev->next = tmpstr->next;
         else
           userstruct = tmpstr->next;

         if (tmpstr->next)
           tmpstr->next->prev = tmpstr->prev;
         MyFree( tmpstr->userhost );
	 userstrcnt--;	
         putlog( LOG1, "Deleted, decrementing counter to %d",
                 userstrcnt );
     }  
      
}

#ifdef USER_COMMANDS
void
onstats( char* who)
{
    char inbuf[256];

    memset( &inbuf, 0x00, sizeof(inbuf));

    sprintf( inbuf, "SocksCleaner Statistic information" );
    toserv(":%s PRIVMSG %s :%s\n",
                  CleanerName,
                  who,
                  inbuf );	
    sprintf( inbuf, "Hosts checked     :%d", iChecked);
    toserv(":%s PRIVMSG %s :%s\n",
                  CleanerName,
                  who,
                  inbuf );
    sprintf( inbuf, "Hosts glined      :%d", iGlined);
    toserv(":%s PRIVMSG %s :%s\n",
                  CleanerName,
                  who,
                  inbuf );
    sprintf( inbuf, "Hosts protected   :%d", iProtected);
    toserv(":%s PRIVMSG %s :%s\n",
                  CleanerName,
                  who,
                  inbuf );
    sprintf( inbuf, "Thread in process :%d", iThreads);
    toserv(":%s PRIVMSG %s :%s\n",
                  CleanerName,
                  who,
                  inbuf );

     if( EnableMonitoring ) {
           sprintf( inbuf, "Requesting statistical info from %s",
                  who );
           toserv(":%s PRIVMSG %s :%s\n",
                  CleanerName,
                  MonitoringChannel,
                  inbuf );
      } /* enable monitoring */

    return;
}

void
oncheck( char* who, char* command)
{
    char inbuf[256];
  struct Luser lptr;

#ifdef HAVE_PTHREADS
      pthread_t clientid;
#endif

    memset( &lptr   , 0x00, sizeof(lptr)  );
    memset( &inbuf, 0x00, sizeof(inbuf));

    sprintf(lptr.nick     , "system");
    sprintf(lptr.username , "system");
    strcpy(lptr.hostname  , command );

#ifdef HAVE_PTHREADS
      pthread_create(&clientid, NULL, (void *) &Proxy, (void *) &lptr);
      pthread_detach(clientid);
      pthread_join(clientid,NULL);
      usleep(10);
#endif /* HAVE_PTHREADS */

    sprintf( inbuf, "Performing check for host:%s", command );
    toserv(":%s PRIVMSG %s :%s\n",
                  CleanerName,
                  who,
                  inbuf );
     if( EnableMonitoring ) {
           sprintf( inbuf, "Forcing check for host:%s from %s",
                  command,
                  who );
           toserv(":%s PRIVMSG %s :%s\n",
                  CleanerName,
                  MonitoringChannel,
                  inbuf );
      } /* enable monitoring */
}

#endif /* USER_COMMANDS */
