/*
 * SocksCleaner TS Services, Copyright (C) 2000 Nedelcho Stanev
 * This program comes with absolutely NO WARRANTY
 *
 * Should you choose to use and/or modify this source code, please
 * do so under the terms of the GNU General Public License under which
 * this program is distributed.
 *
 * $Id: settings.c,v 1.0.0 2000/03/03 22:17:23 decho Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include "alloc.h"
#include "config.h"
#include "scdefs.h"
#include "log.h"
#include "match.h"
#include "misc.h"
#include "settings.h"
#include "timestr.h"

/*
 * These variables, or directives, are set by parsing
 * SETPATH (settings.conf)
 */

char      *HPath;
char      *ConfigFile;
char      *LogFile;
char      *PidFile;
char      *GlineReason;
char      *KillReason;
char      *CleanerName;
char      *MonitoringChannel;
char	  *CheckNotifyMsg;

#ifdef USE_SERVICES
char      *ServicesHost;
#endif

long      ConnectFrequency;
long      ConnectBurst;
long      ServerBurst;
int       ProxyCheck;
int       MaxProxies;
int       KillSocks;
int       GlineSocks;
int       LogLevel;
int       MaxLogs;
int       EnableMonitoring;

static void ClearDirectives();
static int CheckDirectives();
static int dparse(char *, int, int);

struct Directive directives[] = {

  /* File Paths */
  { "HPath", D_NORUNTIME,           { { PARAM_STRING, &HPath } } },
  { "ConfigFile", D_NORUNTIME,      { { PARAM_STRING, &ConfigFile } } },
  { "LogFile", D_NORUNTIME,         { { PARAM_STRING, &LogFile } } },
  { "PidFile", D_NORUNTIME,         { { PARAM_STRING, &PidFile } } },
  /* Psuedo-Client Nicknames/Idents/Descriptions and Options */

#ifdef USE_SERVICES
  { "ServicesHost",  D_OPTIONAL,      { { PARAM_STRING, &ServicesHost } } },
#endif

  /* General Configuration */
  { "ConnectFrequency", D_REQUIRED, { { PARAM_TIME, &ConnectFrequency } } },
  { "ConnectBurst", D_REQUIRED,     { { PARAM_TIME, &ConnectBurst } } },
  { "ServerBurst", D_REQUIRED,      { { PARAM_TIME, &ServerBurst } } },
  { "ProxyCheck", D_OPTIONAL,       { { PARAM_SET, &ProxyCheck } } },
  { "KillSocks", D_OPTIONAL,        { { PARAM_SET, &KillSocks } } } ,
  { "GlineSocks", D_OPTIONAL,       { { PARAM_SET, &GlineSocks } } } ,
  { "GlineReason", D_OPTIONAL,      { { PARAM_STRING, &GlineReason } } },
  { "KillReason",  D_OPTIONAL,      { { PARAM_STRING, &KillReason } } },
  { "CleanerName", D_REQUIRED,      { { PARAM_STRING, &CleanerName } } },
  { "EnableMonitoring", D_OPTIONAL, { { PARAM_INT, &EnableMonitoring } } },
  { "CheckNotifyMsg", D_OPTIONAL,   { { PARAM_STRING, &CheckNotifyMsg } } } ,
  { "MonitoringChannel", D_OPTIONAL,{ { PARAM_STRING, &MonitoringChannel } } },
  { "MaxProxies", D_OPTIONAL,       { { PARAM_INT, &MaxProxies } } },
  { "LogLevel", D_REQUIRED,         { { PARAM_INT, &LogLevel } } },
  { "MaxLogs", D_OPTIONAL,          { { PARAM_INT, &MaxLogs } } },
  { 0, 0,                           { { 0, 0 } } }
};

/*
ClearDirectives()
 Zero all of the ptr fields of directives[]

If rehash is 1, zero all directives that are NOT required. If
we zero a required directive, and they commented it out, it would
never get reset, which could cause a seg fault down the line. This
way, if they comment out a required directive and rehash, we still
have the old copy to work from. If they change a required directive's
value and rehash, it will simply get set to the new value, so
everything is fine.
*/

static void
ClearDirectives(int rehash)

{
  struct Directive *tmp;
  int ii;

  for (tmp = directives; tmp->name; ++tmp)
  {
    if (rehash &&
        ((tmp->flag == D_NORUNTIME) || (tmp->flag == D_REQUIRED)))
      continue;

    for (ii = 0; ii < PARAM_MAX; ii++)
    {
      if (!tmp->param[ii].type)
        break;

      switch (tmp->param[ii].type)
      {
        case PARAM_STRING:
        {
          *(char **) tmp->param[ii].ptr = (char *) NULL;
          break;
        }

        case PARAM_TIME:
        {
          *(long **) tmp->param[ii].ptr = (long *) NULL;
          break;
        }

        case PARAM_INT:
        case PARAM_SET:
        case PARAM_PORT:
        {
          *(int *) tmp->param[ii].ptr = (int) NULL;
          break;
        }

        default: break;
      } /* switch (tmp->param[ii].type) */
    }
  }
} /* ClearDirectives() */

/*
CheckDirectives()
 Make sure all required directives were specified (by checking
if any of them have zero values)
Return 1 if they are all ok, 0 if not
*/

static int
CheckDirectives()

{
  struct Directive *tmp;
  int ii,
      ret,
      good;

  ret = 1;
  for (tmp = directives; tmp->name; tmp++)
  {
    if ((tmp->flag == D_REQUIRED) || (tmp->flag == D_NORUNTIME))
    {
      for (ii = 0; ii < PARAM_MAX; ii++)
      {
        if (!tmp->param[ii].type)
          break;

        good = 1;
        switch (tmp->param[ii].type)
        {
          case PARAM_STRING:
          {
            if (!(*(char **) tmp->param[ii].ptr))
              good = 0;

            break;
          }

          case PARAM_TIME:
          {
            if (!(*(long **) tmp->param[ii].ptr))
              good = 0;

            break;
          }

          case PARAM_INT:
          case PARAM_SET:
          case PARAM_PORT:
          {
            if (!(*(int *) tmp->param[ii].ptr))
              good = 0;

            break;
          }

          default:
          {
            /* shouldn't get here */
            good = 0;
            break;
          }
        }

        if (!good)
        {
          ret = 0;
          fatal(1, "%s: Missing required directive [%s]",
            SETPATH,
            tmp->name);
        }
      }
    }
  }

  if (ProxyCheck && !MaxProxies)
  {
    fatal(1,
      "%s: When ProxyCheck is enabled, MaxProxies is also required",
      SETPATH);
    ret = 0;
  }
  
  if( EnableMonitoring && !MonitoringChannel ) {
   fatal(1,
      "%s: When EnableMonitoring is enabled, MonitoringChannel is also required",
      SETPATH);
    ret = 0;
  }

  return (ret);
} /* CheckDirectives() */

/*
FindDirective()
 Return pointer to index of directives[] matching 'directive'
*/

struct Directive *
FindDirective(char *directive)

{
  struct Directive *tmp;

  if (!directive)
    return (NULL);

  for (tmp = directives; tmp->name; tmp++)
    if (!strcasecmp(directive, tmp->name))
      return (tmp);

  return (NULL);
} /* FindDirective() */

/*
dparse()
 Parse the given line from the settings file SETPATH
Return: 2 if the line is ok
        1 if the line contained warnings
        0 if the line contained errors
*/

static int
dparse(char *line, int linenum, int rehash)

{
  struct Directive *dptr;
  int larc; /* line arg count */
  char *larv[PARAM_MAX]; /* holds line arguements */
  char *lineptr,
       *tmp;
  int ret,
      ii,
      pcnt;

  ret = 2;

  /*
   * We can't use SplitBuf() to break up the line, since
   * it doesn't handle quotes etc. - do it manually
   */

  larc = 0;
  for (lineptr = line; *lineptr; lineptr++)
  {
    if (larc >= PARAM_MAX)
    {
      fatal(1, "%s:%d Too many parameters (max: %d)",
        SETPATH,
        linenum,
        PARAM_MAX);
      ret = 0;
      break;
    }

    if (!lineptr || !*lineptr)
      break;

    while (*lineptr && IsSpace(*lineptr))
      lineptr++;

    if (!*lineptr)
      break;

    tmp = lineptr;

    if (*lineptr == '"')
    {
      /* we encountered a quote */

      /* advance past quotation mark */
      tmp++;
      lineptr++;

      /*
       * Now advance lineptr until we hit another quotation mark,
       * or the end of the line
       */
      while (*lineptr && (*lineptr != '"'))
      {
        /*
         * If they want to include double quotes, they can
         * put a \" inside the quoted string, so advance
         * lineptr past it twice, once for the '\' and once
         * for the '"'.
         */
        if (*lineptr == '\\')
          lineptr++;

        lineptr++;
      }

      if (!*lineptr)
      {
        /* we hit EOL without reaching another quotation mark */
        fatal(1, "%s:%d Unterminated quote",
          SETPATH,
          linenum);
        ret = 0;
        break;
      }

      *lineptr = '\0';
    } /* if (*lineptr == '"') */
    else
    {
      /*
       * Advance lineptr until we hit a space
       */
      while (*lineptr && !IsSpace(*lineptr))
        lineptr++;

      if (*lineptr)
        *lineptr = '\0';
    }

    larv[larc++] = tmp;
  } /* for (lineptr = line; *lineptr; lineptr++) */

  if (!ret || !larc)
    return (ret);

  if (!(dptr = FindDirective(larv[0])))
  {
    fatal(1, "%s:%d Unknown directive [%s] (ignoring)",
      SETPATH,
      linenum,
      larv[0]);
    return (1);
  }

  if (rehash && (dptr->flag == D_NORUNTIME))
  {
    /*
     * SETPATH is being parsed during a rehash, and we
     * hit a non-run time configurable option, ignore it
     */
    return (2);
  }

  pcnt = 1;
  for (ii = 0; ii < PARAM_MAX; ii++, pcnt++)
  {
    if (!dptr->param[ii].type)
      break;

    /*
     * Now assign our variable (dptr->param[ii].ptr) to the
     * corresponding larv[] slot
     */
    if ((dptr->param[ii].type != PARAM_SET) && (pcnt >= larc))
    {
      fatal(1, "%s:%d Not enough arguements to [%s] directive",
        SETPATH,
        linenum,
        dptr->name);
      return (0);
    }

    switch (dptr->param[ii].type)
    {
      case PARAM_STRING:
      {
        if ((*(char **) dptr->param[ii].ptr) != (char *) NULL)
        {
          /*
           * If this is a rehash, make sure we free the old
           * string before allocating a new one. If this
           * is NOT a rehash, we should never get here, because
           * ClearDirectives() would have already zeroed
           * all ptr fields of directives[]
           */
          MyFree(*(char **) dptr->param[ii].ptr);
        }

        *(char **) dptr->param[ii].ptr = MyStrdup(larv[pcnt]);
        break;
      }

      case PARAM_INT:
      {
        int value;

        value = IsNum(larv[pcnt]);
        if (!value)
        {
          fatal(1, "%s:%d Invalid integer",
            SETPATH,
            linenum);
          ret = 0;
          break;
        }

        /*
         * We have to call atoi() anyway since IsNum() would
         * have returned 1 if larv[pcnt] was "0", but IsNum()
         * is a good way of making sure we have a valid integer
         */
        *(int *) dptr->param[ii].ptr = atoi(larv[pcnt]);
        break;
      }

      case PARAM_TIME:
      {
        long value;

        value = timestr(larv[pcnt]);
        if (!value)
        {
          fatal(1, "%s:%d Invalid time format",
            SETPATH,
            linenum);
          ret = 0;
          break;
        }

        *(long *) dptr->param[ii].ptr = value;
        break;
      }

      case PARAM_SET:
      {
        *(int *) dptr->param[ii].ptr = 1;
        break;
      }

      case PARAM_PORT:
      {
        int value;

        value = IsNum(larv[pcnt]);
        if (!value)
        {
          fatal(1, "%s:%d Invalid port number (must be between 1 and 65535)",
            SETPATH,
            linenum);
          ret = 0;
          break;
        }

        value = atoi(larv[pcnt]);

        if ((value < 1) || (value > 65535))
        {
          fatal(1, "%s:%d Invalid port number (must be between 1 and 65535)",
            SETPATH,
            linenum);
          ret = 0;
          break;
        }

        *(int *) dptr->param[ii].ptr = value;

        break;
      }

      default:
      {
        /* we shouldn't get here */
        fatal(1, "%s:%d Unknown parameter type [%d] for directive [%s]",
          SETPATH,
          linenum,
          dptr->param[ii].type,
          dptr->name);
        ret = 0;
        break;
      }
    } /* switch (dptr->param[ii].type) */
  } /* for (ii = 0; ii < PARAM_MAX; ii++, pcnt++) */

  return (ret);
} /* dparse() */

/*
LoadSettings()
 Load directives in SETPATH, and make sure all required
directives are specified. Return 1 if successful, 0 if not
 If (rehash == 1), it indicates this function is being
called from a rehash, so don't call ClearDirectives() or
CheckDirectives()
*/

int
LoadSettings(int rehash)

{
  FILE *fp;
  char buffer[MAXLINE];
  int goodread;
  int cnt;

  putlog( LOG1, "Using conf gile:%s", SETPATH);
  if ((fp = fopen(SETPATH, "r")) == (FILE *) NULL)
  {
    fprintf(stderr, "Unable to open SETPATH (%s)\n",
      SETPATH);

    if (rehash)
      return (0);
    else
      exit(0);
  }

  /*
   * If rehash is 0, clear all directives, otherwise clear
   * only PARAM_SET directives in case they comment out a
   * SET directive, in which case we wouldn't be able to
   * detect it, unless it was already zero'd
   */
  ClearDirectives(rehash);

  cnt = 0;
  goodread = 1;

  while (fgets(buffer, sizeof(buffer), fp))
  {
    cnt++;
    if (buffer[0] != '#') /* comment line */
    {
      int tmp;

      tmp = dparse(buffer, cnt, rehash);

      if ((goodread > 0) && (tmp == 0))
        goodread = 0;
    }
  }

  fclose(fp);

  if (!rehash)
  {
    /*
     * Now check to make sure all required directives
     * were specified
     */
    if (CheckDirectives() == 0)
      goodread = 0;
  }

  return (goodread);
} /* LoadSettings() */

